
//******************************************************************************
// RCF - Remote Call Framework
// Copyright (c) 2005 - 2008, Jarl Lindrud. All rights reserved.
// Consult your license for conditions of use.
// Version: 1.0
// Contact: jarl.lindrud <at> gmail.com 
//******************************************************************************

#ifndef INCLUDE_RCF_CONNECTIONORIENTEDCLIENTTRANSPORT_HPP
#define INCLUDE_RCF_CONNECTIONORIENTEDCLIENTTRANSPORT_HPP

#include <RCF/AsyncFilter.hpp>
#include <RCF/ByteOrdering.hpp>
#include <RCF/ClientProgress.hpp>
#include <RCF/ClientTransport.hpp>
#include <RCF/Export.hpp>

namespace RCF {

    class ConnectionOrientedClientTransport;

    class ClientFilterProxy;

    class RCF_EXPORT ConnectionOrientedClientTransport : 
        public I_ClientTransport, 
        public WithProgressCallback
    {
    public:
        typedef boost::function0<void> CloseFunctor;
        typedef boost::function0<void> NotifyCloseFunctor;

        ConnectionOrientedClientTransport(const ConnectionOrientedClientTransport &rhs);
        ConnectionOrientedClientTransport();
        ~ConnectionOrientedClientTransport();

        void                    setCloseFunctor(const CloseFunctor &closeFunctor);
        void                    setNotifyCloseFunctor(const NotifyCloseFunctor &notifyCloseFunctor);
        void                    close();
        void                    setMaxSendSize(std::size_t maxSendSize);
        std::size_t             getMaxSendSize();

    private:

        void                    read(const ByteBuffer &byteBuffer_, std::size_t bytesRequested);
        void                    write(const std::vector<ByteBuffer> &byteBuffers);
        std::size_t             timedSend(const std::vector<ByteBuffer> &data);
        std::size_t             timedReceive(ByteBuffer &byteBuffer, std::size_t bytesRequested);
        int                     send(const std::vector<ByteBuffer> &data, unsigned int timeoutMs);
        int                     receive(ByteBuffer &byteBuffer, unsigned int timeoutMs);

        void                    setTransportFilters(const std::vector<FilterPtr> &filters);
        void                    getTransportFilters(std::vector<FilterPtr> &filters);
        void                    connectTransportFilters();
        
        void                    connect(unsigned int timeoutMs);
        void                    disconnect(unsigned int timeoutMs);
        int                     timedSend(const char *buffer, std::size_t bufferLen);
        int                     timedReceive(char *buffer, std::size_t bufferLen);

    protected:

        void                    onReadCompleted(const ByteBuffer &byteBuffer, int error);
        void                    onWriteCompleted(std::size_t bytes, int error);

        bool                    mOwn;
        bool                    mClosed;
        std::size_t             mMaxSendSize;
        std::size_t             mBytesTransferred;
        std::size_t             mBytesSent;
        std::size_t             mBytesRead;
        std::size_t             mBytesTotal;
        int                     mError;
        unsigned int            mEndTimeMs;

        CloseFunctor                            mCloseFunctor;
        NotifyCloseFunctor                      mNotifyCloseFunctor;
        std::vector<FilterPtr>                  mTransportFilters;
        std::vector<ByteBuffer>                 mByteBuffers;
        std::vector<ByteBuffer>                 mSlicedByteBuffers;
        boost::shared_ptr<std::vector<char> >   mReadBufferPtr;
        boost::shared_ptr<std::vector<char> >   mReadBuffer2Ptr;

        friend class ClientFilterProxy;

    private:

        virtual std::size_t implRead(
            const ByteBuffer &byteBuffer_,
            std::size_t bytesRequested) = 0;

        virtual std::size_t implWrite(const std::vector<ByteBuffer> &byteBuffers) = 0;

        virtual void implConnect(unsigned int timeoutMs) = 0;

        virtual void implClose() = 0;

    };

} // namespace RCF

#endif // ! INCLUDE_RCF_CONNECTIONORIENTEDCLIENTTRANSPORT_HPP
