
//******************************************************************************
// RCF - Remote Call Framework
// Copyright (c) 2005 - 2008, Jarl Lindrud. All rights reserved.
// Consult your license for conditions of use.
// Version: 1.0
// Contact: jarl.lindrud <at> gmail.com 
//******************************************************************************

#ifndef INCLUDE_RCF_WIN32NAMEDPIPESERVERTRANSPORT_HPP
#define INCLUDE_RCF_WIN32NAMEDPIPESERVERTRANSPORT_HPP

#include <RCF/Export.hpp>
#include <RCF/ServerTransport.hpp>
#include <RCF/Service.hpp>
#include <RCF/TcpIocpServerTransport.hpp>

#include <RCF/util/Tchar.hpp>
#include "tchar.h"

namespace RCF {

    class Win32NamedPipeServerTransport;

    class RCF_EXPORT Win32NamedPipeImpersonator
    {
    public:
        Win32NamedPipeImpersonator();
        ~Win32NamedPipeImpersonator();
        void impersonate();
        void revertToSelf() const;
    };

    class RCF_EXPORT Win32NamedPipeOverlapped : public OVERLAPPED, public I_Proactor
    {
    public:

        friend class Win32NamedPipeImpersonator;

        Win32NamedPipeOverlapped(HANDLE hPipe, Win32NamedPipeServerTransport & serverTransport);
        ~Win32NamedPipeOverlapped();

        enum State
        {
            Connecting,
            Reading,
            Writing
        };

        void reconnect();
        void issueRead();
        void issueWrite();
        void onCompletion(std::size_t bytesTransferred);
        HANDLE getNativeHandle() const;

    private:

        HANDLE                                  mhPipe;
        State                                   mState;
        boost::shared_ptr< std::vector<char> >  mReadBufferPtr;
        std::vector<ByteBuffer>                 mWriteBuffers;           
        std::size_t                             mReadBufferPos;
        std::size_t                             mWriteBufferPos;
        Win32NamedPipeServerTransport &         mServerTransport;
        SessionPtr                              mSessionPtr;
        NoRemoteAddress                         mRemoteAddress;

        // I_Proactor members
        void                            postRead();
        ByteBuffer                      getReadByteBuffer();
        void                            postWrite(const std::vector<ByteBuffer> &byteBuffers);
        void                            postClose();
        I_ServerTransport &             getServerTransport();
        const I_RemoteAddress &         getRemoteAddress();
        void                            setTransportFilters(const std::vector<FilterPtr> &filters);
        void                            getTransportFilters(std::vector<FilterPtr> &);        
    };

    typedef Win32NamedPipeOverlapped Win32NamedPipeProactor;

    class RCF_EXPORT Win32NamedPipeServerTransport : 
        public I_ServerTransport,
        public I_Service,
        boost::noncopyable
    {
    private:
        friend class Win32NamedPipeImpersonator;
        friend class Win32NamedPipeOverlapped;

    public:
        Win32NamedPipeServerTransport(
            const tstring & pipeName, 
            const tstring & machine = RCF_T(""));

        ~Win32NamedPipeServerTransport();
        ServerTransportPtr clone();
        bool cycleTransportAndServer(RcfServer &server, int timeoutMs, const volatile bool &stopFlag);
        void cycle(int timeoutMs, const volatile bool &stopFlag);
        void open();
        void close();

        void setSecurityAttributes(LPSECURITY_ATTRIBUTES pSec);

        // I_Service implementation
    private:
        void onServiceAdded(RcfServer &server);
        void onServiceRemoved(RcfServer &server);
        void onServerOpen(RcfServer &server);
        void onServerClose(RcfServer &server);
        void onServerStart(RcfServer &server);
        void onServerStop(RcfServer &server);

    private:
        tstring                                     mPipeName;
        I_SessionManager *                          mpSessionManager;
        bool                                        mStopFlag;
        std::vector<Win32NamedPipeOverlapped *>                   mPipes;
        boost::scoped_ptr<RCF::TcpIocp::Iocp>       mIocp;
        LPSECURITY_ATTRIBUTES                       mpSec;
    };

} // namespace RCF

#endif // ! INCLUDE_RCF_WIN32NAMEDPIPESERVERTRANSPORT_HPP
