
//******************************************************************************
// RCF - Remote Call Framework
// Copyright (c) 2005 - 2008, Jarl Lindrud. All rights reserved.
// Consult your license for conditions of use.
// Version: 1.0
// Contact: jarl.lindrud <at> gmail.com 
//******************************************************************************

#include <RCF/Tools.hpp>

#include <RCF/InitDeinit.hpp>

#include <RCF/util/Platform/OS/GetCurrentTime.hpp>

namespace RCF {

    util::TraceChannel *pTraceChannels[10];

    util::TraceChannel **getTraceChannels()
    {
        return pTraceChannels;
    }

    // current time in ms, modulo 65536 s (turns over every ~18.2 hrs)
    unsigned int getCurrentTimeMs()
    {
        return Platform::OS::getCurrentTimeMs();
    }

    // Generate a timeout value for the given ending time.
    // Returns zero if endTime <= current time <= endTime+10% of timer resolution,
    // otherwise returns a nonzero duration in ms.
    // Timer resolution as above (18.2 hrs).
    unsigned int generateTimeoutMs(unsigned int endTimeMs)
    {
        // 90% of the timer interval
        static const unsigned int maxTimeoutMs = (((unsigned int)-1)/10)*9;
        unsigned int currentTimeMs = getCurrentTimeMs();
        unsigned int timeoutMs = endTimeMs - currentTimeMs;
        return (timeoutMs < maxTimeoutMs) ? timeoutMs : 0;
    }

    void deinitRcfTraceChannels()
    {
        for (int i=0; i<10; ++i)
        {
            delete pTraceChannels[i];
            pTraceChannels[i] = NULL;
        }
    }

    void initRcfTraceChannels()
    {
        deinitRcfTraceChannels();
        for (int i=0; i<10; ++i)
        {
            std::string name = "RCF";
            if (i > 0)
            {
                name += '0' + static_cast<char>(i);
            }
            pTraceChannels[i] = new util::TraceChannel(name);
        }

#ifndef NDEBUG
        pTraceChannels[9]->setTraceTarget("ODS");
#endif

    }

    RCF_ON_INIT_DEINIT_NAMED(
        initRcfTraceChannels(),
        deinitRcfTraceChannels(),
        InitRcfTraceChannels);

} // namespace RCF
