
//******************************************************************************
// RCF - Remote Call Framework
// Copyright (c) 2005 - 2008, Jarl Lindrud. All rights reserved.
// Consult your license for conditions of use.
// Version: 1.0
// Contact: jarl.lindrud <at> gmail.com 
//******************************************************************************

#include <RCF/Win32NamedPipeEndpoint.hpp>

#include <RCF/InitDeinit.hpp>
#include <RCF/Win32NamedPipeClientTransport.hpp>
#include <RCF/Win32NamedPipeServerTransport.hpp>

#ifdef RCF_USE_SF_SERIALIZATION
#include <SF/Registry.hpp>
#endif

#include <boost/lexical_cast.hpp>

namespace RCF {

    Win32NamedPipeEndpoint::Win32NamedPipeEndpoint()
    {}

    Win32NamedPipeEndpoint::Win32NamedPipeEndpoint(
        const tstring & pipeName, 
        const tstring & machine) :
            mPipeName(pipeName),
            mMachine(machine)
    {}

    ServerTransportAutoPtr Win32NamedPipeEndpoint::createServerTransport() const
    {
        return ServerTransportAutoPtr(
            new Win32NamedPipeServerTransport(mPipeName, mMachine));
    }

    ClientTransportAutoPtr Win32NamedPipeEndpoint::createClientTransport() const
    {            
        return ClientTransportAutoPtr(
            new Win32NamedPipeClientTransport(mPipeName, mMachine));
    }

    EndpointPtr Win32NamedPipeEndpoint::clone() const
    {
        return EndpointPtr( new Win32NamedPipeEndpoint(*this) );
    }

    tstring generateNewPipeName()
    {
        tstring pipePrefix = RCF_T("\\\\.\\pipe\\RcfTestPipe_");
        unsigned int i = 0;
        while (true)
        {
            tstring candidateName = 
                pipePrefix + boost::lexical_cast<tstring>(++i);

            HANDLE hPipe = CreateFile( 
                candidateName.c_str(),   // pipe name 
                GENERIC_READ |  // read and write access 
                GENERIC_WRITE, 
                0,              // no sharing 
                NULL,           // default security attributes
                OPEN_EXISTING,  // opens existing pipe 
                0,              // default attributes 
                NULL);          // no template file 

            DWORD dwErr = GetLastError();
            if (hPipe == INVALID_HANDLE_VALUE && dwErr != ERROR_PIPE_BUSY)
            {
                return candidateName;
            }
        }
    }

    inline void initWin32NamedPipeEndpointSerialization()
    {
#ifdef RCF_USE_SF_SERIALIZATION
        SF::registerType( (Win32NamedPipeEndpoint *) 0, "RCF::Win32NamedPipeEndpoint");
        SF::registerBaseAndDerived( (I_Endpoint *) 0, (Win32NamedPipeEndpoint *) 0);
#endif
    }

    RCF_ON_INIT_NAMED( initWin32NamedPipeEndpointSerialization(), InitWin32NamedPipeEndpointSerialization );

} // namespace RCF
