
//******************************************************************************
// RCF - Remote Call Framework
// Copyright (c) 2005 - 2008, Jarl Lindrud. All rights reserved.
// Consult your license for conditions of use.
// Version: 1.0
// Contact: jarl.lindrud <at> gmail.com 
//******************************************************************************

#include <RCF/util/Trace.hpp>

#include <fstream>
#include <iostream>
#include <sstream>

#include <RCF/util/Platform/OS/OutputDebugString.hpp>

namespace util {

    std::string TraceTargetNull::getName() 
    { 
        return "null"; 
    }
    
    bool TraceTargetNull::isNull() 
    { 
        return true; 
    }

    void TraceTargetNull::trace(const std::string &msg) 
    { 
        RCF_UNUSED_VARIABLE(msg); 
    }

    std::string TraceTargetOds::getName()
    {
        return "ODS";
    }

    bool TraceTargetOds::isNull()
    {
        return false;
    }
    
    void TraceTargetOds::trace(const std::string &msg)
    {
        Lock lock(m); RCF_UNUSED_VARIABLE(lock);
        Platform::OS::OutputDebugString( msg.c_str() );
    }

    std::string TraceTargetStdout::getName()
    {
        return "stdout";
    }

    bool TraceTargetStdout::isNull()
    {
        return false;
    }

    void TraceTargetStdout::trace(const std::string &msg)
    {
        Lock lock(m); RCF_UNUSED_VARIABLE(lock);
        std::cout << msg;
        std::cout.flush();
    }
    
    std::string TraceTargetStderr::getName()
    {
        return "stderr";
    }

    bool TraceTargetStderr::isNull()
    {
        return false;
    }

    void TraceTargetStderr::trace(const std::string &msg)
    {
        Lock lock(m); RCF_UNUSED_VARIABLE(lock);
        std::cerr << msg;
        std::cerr.flush();
    }

    TraceTargetFile::TraceTargetFile( const std::string &filename ) :
        fout( new std::ofstream(filename.c_str())),
        filename(filename)
    {}

    bool TraceTargetFile::isNull()
    {
        return false;
    }

    std::string TraceTargetFile::getName()
    {
        return filename;
    }

    void TraceTargetFile::trace(const std::string &msg)
    {
        Lock lock(m); RCF_UNUSED_VARIABLE(lock);
        *fout << msg;
        fout->flush();
    }
    
} // namespace util
