
// uncomment to enable VLD leak detection - will automatically link to required libs
//#include "vld.h"
//#include "vldapi.h"

#include <string>

#include <RCF/test/TestMinimal.hpp>

#include <RCF/Idl.hpp>
#include <RCF/RcfServer.hpp>
#include <RCF/test/EndpointFactories.hpp>
#include <RCF/test/TransportFactories.hpp>
#include <RCF/test/WithStopServer.hpp>
#include <RCF/util/CommandLine.hpp>

#include <SF/wstring.hpp>

class Echo : public RCF::WithStopServer
{
public:
    //Echo()
    //{
    //    RCF::Thread( boost::bind(&Echo::setLock, this)).join();
    //}

    std::string echo(const std::string &s)
    {
        std::cout << "Echoing: " << s << std::endl;
        return s;
    }

    std::wstring echo(const std::wstring &s)
    {
        return s;
    }

    /*void stopServer()
    {
        mLockPtr->unlock();
    }

    void wait()
    {
        RCF::Lock lock(mMutex);
    }*/

private:

   /* void setLock()
    {
        mLockPtr.reset( new RCF::Lock(mMutex));
    }

    RCF::Mutex mMutex;
    RCF::LockPtr mLockPtr;
*/
};

RCF_BEGIN(I_Echo, "I_Echo")
    RCF_METHOD_R1(std::string, echo, const std::string &)
    RCF_METHOD_R1(std::wstring, echo, const std::wstring &)
    RCF_METHOD_V0(void, stopServer)
RCF_END(I_Echo)


void myCallback(
    std::size_t bytesTransferred,
    std::size_t bytesTotal,
    RCF::ClientProgress::Trigger trigger,
    RCF::ClientProgress::Activity activity,
    RCF::ClientProgress::Action &action)
{
    action = RCF::ClientProgress::Continue;
}

int test_main(int argc, char **argv)
{
    RCF::RcfInitDeinit rcfInitDeinit;

    util::CommandLineOption<bool>           clServer("server", false, "act as server");
    util::CommandLineOption<bool>           clClient("client", false, "act as client");
    util::CommandLineOption<std::string>    clIp("ip", util::PortNumbers::getSingleton().getIp(), "ip");
    util::CommandLineOption<int>            clPort("port", util::PortNumbers::getSingleton().getCurrent(), "port");
    util::CommandLine::getSingleton().parse(argc, argv);

    util::PortNumbers::getSingleton().setCurrent(clPort);
    util::PortNumbers::getSingleton().setIp(clIp);

    bool bServer = true;
    bool bClient = true;
    if (clServer && clClient)
    {
        bServer = true;
        bClient = true;
    }
    else if (clServer)
    {
        bServer = true;
        bClient = false;
    }
    else if (clClient)
    {
        bServer = false;
        bClient = true;
    }

    RCF::TransportFactoryPtr transportFactoryPtr( new RCF::TcpIocpTransportFactory );
    std::pair<RCF::ServerTransportPtr, RCF::ClientTransportAutoPtrPtr> transports = transportFactoryPtr->createTransports();
    RCF::ServerTransportPtr serverTransportPtr( transports.first );
    RCF::ClientTransportAutoPtr clientTransportAutoPtr( *transports.second );

    serverTransportPtr->setMaxMessageLength(-1);
    clientTransportAutoPtr->setMaxMessageLength(-1);

    Echo echo;
    RCF::RcfServer server(serverTransportPtr);
    server.bind( (I_Echo*) 0, echo);
    if (bServer)
    {
        if (!bClient)
        {
            RCF::I_IpServerTransport &ipServerTransport = dynamic_cast<RCF::I_IpServerTransport &>(*serverTransportPtr);
            ipServerTransport.setNetworkInterface("0.0.0.0");
        }
        server.start();
        BOOST_CHECK( RcfClient<I_Echo>(clientTransportAutoPtr->clone()).echo("asdf") == "asdf");
    }
    if (bClient)
    {
        RcfClient<I_Echo> client(clientTransportAutoPtr->clone());
        client.getClientStub().setRemoteCallTimeoutMs(1000*60*60);

        std::string line;
        while (line != "quit")
        {
            std::cout << "Input string to echo (\"quit\" to quit): ";
            std::getline(std::cin, line);
            std::cout << "Echo: " << client.echo(line) << std::endl;
        }

        RcfClient<I_Echo>(clientTransportAutoPtr->clone()).stopServer();
    }
    if (bServer)
    {
        echo.wait();
        server.stop();
    }

    return boost::exit_success;
}
